#include <IRremote.h>
#define RECV_PIN  8 // Пин для подключения ИК-приёмника
IRrecv irrecv(RECV_PIN);
decode_results results;
// ----------------------------------------
// Коды кнопок с пульта управления
// Данные коды необходимо заменить на свои
#define UP    0x2FD9867
#define DOWN  0x2FDB847
#define RIGHT 0x2FDE21D
#define LEFT  0x2FD629D
#define OK    0x2FD50AF
// ----------------------------------------
// Библиотеки для работы с дисплеем NOKIA_5110
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

// Подключение дисплея к аппаратному SPI Arduino
// SCLK - пин №13
// DIN  - пин №11
// D/C  - пин №5
// CS   - пин №4
// RST  - пин №3
Adafruit_PCD8544 display = Adafruit_PCD8544(13, 11, 5, 4, 3);

#define pinGreenLed 6 // Пин для подключения зелёного светодиода
#define pinRedLed   7 // Пин для подключения красного светодиода
#define pinBuzzer   9 // Пин для подключения буззера

bool flagDraw = true;             // Флаг, разрешающий отрисовку изображения на дисплее
volatile int menuCursor = 0;      // Номер отображаемого в данный момент экрана
volatile uint8_t globalState = 0; // Переменная глобального состояния системы
uint32_t ledBlinkTimer = 0;       // Таймер для отсчёта периода мигания светодиодов

// Макроопределения режимов стрельбы
#define EASY    1
#define MEDIUM  2
#define HARD    3

volatile uint8_t shootMode = 0; // Текущий режим стрельбы
volatile bool shootFlag = false; // Флаг, разрешающий обработку датчика вибрации в прерывании
volatile uint16_t shootScore = 0; // Кол-во баллов/попаданий
uint8_t randomTime = 3; // Случайное время, в течение которого можно поразить мишень в режиме MEDIUM
uint32_t shootTime = 1; // Общее время работы мишени в режиме HARD
int shootInterval[5] = {0, 0, 0, 0, 0}; // Промежутки времени, в течение которых мишень доступна для поражения
int hardModeCursor = 0; // Курсор для настройки разных параметров в режиме HARD
int TRY = 0; // Кол-во попыток, данных в режиме HARD 
volatile int shootIntervalCursor = 0; // Курсор индекса для переключения между интервалами активности мишени


// Переменные для режима MEDIUM
uint32_t mm_randomTime;  // Случайное время в пределах от 3 сек. до RandomTime, установленном в меню
uint32_t mm_randomTimer; // Таймер для отсчёта случайного времени

// Переменные для режима HARD
uint32_t hm_Time;
uint32_t hm_Timer;
uint32_t shootTimer;

/* Функция подачи звукового сигнала низкого тона */
void beep() {
  for(int i = 0; i < 500; i++) {
    digitalWrite(pinBuzzer, HIGH);
    delay(1);
    digitalWrite(pinBuzzer, LOW);
    delay(1); 
  }
}
/* Функция подачи звукового сигнала высокого тона */
void beep_1() {
  for(int i = 0; i < 500; i++) {
    digitalWrite(pinBuzzer, HIGH);
    delayMicroseconds(500);
    digitalWrite(pinBuzzer, LOW);
    delayMicroseconds(500);
  }
}
/* Функция предустановок */
void setup() {
  attachInterrupt(0, shoot, CHANGE); // Внешнее прерывание на выводе D2
  pinMode(pinGreenLed, OUTPUT); digitalWrite(pinGreenLed, LOW);
  pinMode(pinRedLed, OUTPUT); digitalWrite(pinRedLed, LOW);
  pinMode(pinBuzzer, OUTPUT);
  Serial.begin(9600); // Инициализация серийного порта
  irrecv.enableIRIn(); // Инициализация ИК-приёмника
  display.begin();  // Инициализация дисплея
  display.setContrast(45); // Установка контраста дисплея
  display.clearDisplay(); // Очистка дисплея
  display.setTextSize(1); // Установка мелкого шрифта
  display.setTextColor(BLACK); // Чёрный цвет текста
  display.setCursor(18, 0);
  display.println("E-TARGET");
  display.display();
  beep();
}
/* Функция отображения меню */
void showMenu(uint8_t flagMenu) {
  if(flagDraw) {
    display.clearDisplay();
    
    // Выводим одну и ту же надпись для 3-х повторяющихся пунктов
    if(flagMenu < 3) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.drawLine(0, 8, 84, 8, BLACK);
      display.setCursor(4, 10); display.println("SHOOTING MODE");
      display.drawLine(0, 18, 84, 18, BLACK);
    }

    if(flagMenu == 0) {
      display.fillRoundRect(0, 20, 33, 9, 3, BLACK);
      display.setTextColor(WHITE); display.setCursor(5, 21); display.println("EASY");
      display.setTextColor(BLACK);
      display.setCursor(5, 30); display.println("MEDIUM");
      display.setCursor(5, 39); display.println("HARD");
    }
    else if(flagMenu == 10) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(15, 9); display.println("EASY MODE");
      display.setTextColor(BLACK);
      display.setCursor(18, 18); display.println("Press OK");
      display.setTextSize(3);
      display.setCursor(35, 27); display.println("0");
    }
    else if(flagMenu == 100) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(15, 9); display.println("EASY MODE");
      display.setTextColor(BLACK);
      display.setCursor(9, 18); display.println("SHOOT SCORE");
      display.setTextSize(3);
      if(shootScore < 10) display.setCursor(35, 27);
      else if(shootScore > 9 && shootScore < 100) display.setCursor(25, 27);
      else if(shootScore > 99 && shootScore < 1000) display.setCursor(17, 27);
      else if(shootScore > 999 && shootScore < 10000) display.setCursor(9, 27);
      display.println(shootScore);
    }
    else if(flagMenu == 1) {
      display.setTextColor(BLACK);
      display.setCursor(5, 21); display.println("EASY");
      display.fillRoundRect(0, 29, 45, 9, 3, BLACK);
      display.setTextColor(WHITE); display.setCursor(5, 30); display.println("MEDIUM");
      display.setTextColor(BLACK);
      display.setCursor(5, 39); display.println("HARD");
    }
    else if(flagMenu == 11) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(9, 9); display.println("MEDIUM MODE");
      display.setTextColor(BLACK);
      display.setCursor(0, 18); display.print("RANDOM TIME:"); 
      if(randomTime < 10) display.print("0");
      display.print(randomTime);
      display.setTextSize(3);
      display.setCursor(35, 27); display.println("0");
    }
    else if(flagMenu == 110) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(9, 9); display.println("MEDIUM MODE");
      display.setTextColor(BLACK);
      display.setCursor(9, 18); display.println("SHOOT SCORE"); 
      display.setTextSize(3);
      if(shootScore < 10) display.setCursor(35, 27);
      else if(shootScore > 9 && shootScore < 100) display.setCursor(25, 27);
      else if(shootScore > 99 && shootScore < 1000) display.setCursor(17, 27);
      else if(shootScore > 999 && shootScore < 10000) display.setCursor(9, 27);
      display.println(shootScore);
    }
    else if(flagMenu == 2) {
      display.setTextColor(BLACK);
      display.setCursor(5, 21); display.println("EASY");
      display.setCursor(5, 30); display.println("MEDIUM");
      display.fillRoundRect(0, 38, 33, 9, 3, BLACK);
      display.setTextColor(WHITE); display.setCursor(5, 39); display.println("HARD");
    }
    else if(flagMenu == 21) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(15, 9); display.println("HARD MODE");
      display.setTextColor(BLACK);
      display.setCursor(0, 18); display.print("SHOOT TIME:");
      if(shootTime < 10) display.print("00");
      else if(shootTime < 100) display.print("0");
      display.print(shootTime);

      // Выводим на дисплей все 5 параметров SHOOT INTERVAL
      display.setCursor(0, 29); display.print("SHOOT INTERVAL");
      display.setCursor(0, 38);
      for(uint8_t i = 0; i < 5; i++) { 
        if(shootInterval[i] < 10) display.print("0");
        display.print(shootInterval[i]); 
        display.print(" "); 
      }

      // Рисуем курсор под тем параметром, который в данный момент необходимо изменить
      switch(hardModeCursor) {
        case 0: display.drawLine(65, 26, 84, 26, BLACK); break;
        case 1: display.drawLine(0, 46, 10, 46, BLACK); break;
        case 2: display.drawLine(18, 46, 28, 46, BLACK); break;
        case 3: display.drawLine(36, 46, 46, 46, BLACK); break;
        case 4: display.drawLine(54, 46, 64, 46, BLACK); break;
        case 5: display.drawLine(72, 46, 82, 46, BLACK); break;
      }
    }
    else if(flagMenu == 210) {
      display.setTextSize(1);
      display.setTextColor(BLACK);
      display.setCursor(18, 0); display.println("E-TARGET");
      display.fillRoundRect(0, 8, 83, 9, 3, BLACK);
      display.setTextColor(WHITE);
      display.setCursor(15, 9); display.println("HARD MODE");
      display.setTextColor(BLACK);
      display.drawLine(42, 18, 42, 48, BLACK); display.drawLine(41, 18, 41, 48, BLACK);
      display.setCursor(12, 18); display.println("TRY");
      display.setCursor(56, 18); display.println("HIT");
      display.drawLine(0, 26, 84, 26, BLACK);
      display.setTextSize(2);
      display.setCursor(3, 31); 
      if(TRY < 10) display.print("00");
      else if((TRY > 9) && (TRY < 100)) display.print("0");
      display.print(TRY);

      display.setCursor(47, 31); 
      if(shootScore < 10) display.print("00");
      else if((shootScore > 9) && (shootScore < 100)) display.print("0");
      display.println(shootScore);
    }
    flagDraw = false;
    display.display();
  }
}
/* ОСНОВНОЙ ЦИКЛ ПРОГРАММЫ */
void loop() {
  while(1) {
    // Если на ИК-приёмник поступил сигнал
    if (irrecv.decode(&results)) {
      /* Если на пульте ДУ нажата кнопка "ВНИЗ" */
      if(results.value == DOWN) {
        // Навигация по "ГЛАВНОМУ МЕНЮ"
        if(menuCursor < 3) {
          menuCursor++;
          if(menuCursor > 2) menuCursor = 0;
          flagDraw  = true;
        }
        // Уменьшение параметра RANDOM TIME в режиме MEDIUM
        if(menuCursor == 11) {
          randomTime--;
          if(randomTime < 3) randomTime = 99;
          flagDraw = true;
        }
        //  Уменьшение одного из параметров в режиме HARD
        if(menuCursor == 21) {
          if(hardModeCursor == 0) { // Уменьшаем параметр SHOOT TIME
            shootTime--;
            if(shootTime < 1) shootTime = 999;
          }
          else if(hardModeCursor > 0) { // Уменьшаем один из параметров SHOOT INTERVAL
            shootInterval[hardModeCursor - 1]--;
            if(shootInterval[hardModeCursor - 1] < 0) shootInterval[hardModeCursor - 1] = 99;
          }
          flagDraw = true;
        }
      }
      /* Если на пульте ДУ нажата кнопка "ВВЕРХ" */
      else if(results.value == UP) {
        // Навигация по "ГЛАВНОМУ МЕНЮ"
        if(menuCursor < 3) {
          menuCursor--;
          if(menuCursor < 0) menuCursor = 2;
          flagDraw  = true;
        }
        // Увеличение параметра RANDOM TIME в режиме MEDIUM
        if(menuCursor == 11) {
          randomTime++;
          if(randomTime > 99) randomTime = 3;
          flagDraw = true;
        }
        //  Увеличение одного из параметров в режиме HARD
        if(menuCursor == 21) {
          if(hardModeCursor == 0) { // Увеличиваем параметр SHOOT TIME
            shootTime++;
            if(shootTime > 999) shootTime = 0;
          }
          else if(hardModeCursor > 0) { // Увеличиваем один из параметров SHOOT INTERVAL
            shootInterval[hardModeCursor - 1]++;
            if(shootInterval[hardModeCursor - 1] > 99) shootInterval[hardModeCursor - 1] = 0;
          }
          flagDraw = true;
        }
      }
      /* Если на пульте ДУ нажата кнопка "ВПРАВО" */
      else if(results.value == RIGHT) {
        // Перемещаем курсор к следующему параметру в режиме HARD
        if(menuCursor == 21) {
          hardModeCursor++;
          if(hardModeCursor > 5) hardModeCursor = 0;
          flagDraw = true;
        }
      }
      /* Если на пульте ДУ нажата кнопка "ВЛЕВО" */
      else if(results.value == LEFT) {
        // Перемещаем курсор к предыдущему параметру в режиме HARD
        if(menuCursor == 21) {
          hardModeCursor--;
          if(hardModeCursor < 0) hardModeCursor = 5;
          flagDraw = true;
        }
      }
      /* Если на пульте ДУ нажата кнопка "ОК" */
      else if(results.value == OK) {
        /* Выход из режима EASY, MEDIUM или HARD*/
        if(menuCursor >= 100) { 
          menuCursor = -1; 
          flagDraw  = true; 
          globalState = 0;
          shootMode = 0;
          shootScore = 0;
          shootFlag = false; 
        }
        /* Нажимаем на пульте ОК. Бузер издает сигнал на секунду. Включается красный светодиод
          (нет попадания) */
        if(menuCursor == 10) { 
          menuCursor = 100; 
          flagDraw  = true; 
          digitalWrite(pinGreenLed, LOW); digitalWrite(pinRedLed, HIGH);
          beep();
          shootMode = EASY;
          globalState = 3; shootFlag = true;
        }
        /* Стреляем в режиме MEDIUM */
        if(menuCursor == 11) {
          menuCursor = 110;
          flagDraw  = true;
          digitalWrite(pinGreenLed, LOW); digitalWrite(pinRedLed, LOW);
          beep();
          shootMode = MEDIUM;
          globalState = 5; shootFlag = true;
        }
        /* Стреляем в режиме HARD */
        if(menuCursor == 21) {
          menuCursor = 210;
          flagDraw  = true;
          digitalWrite(pinGreenLed, LOW); digitalWrite(pinRedLed, LOW);
          beep();
          shootMode = HARD;
          globalState = 7;
          shootTimer = millis(); shootFlag = true;
        }
        // Вход в режим EASY
        if(menuCursor == 0) { menuCursor = 10; shootScore = 0; flagDraw  = true; }
        // Вход в режим MEDIUM
        if(menuCursor == 1) { menuCursor = 11; shootScore = 0; flagDraw  = true; }
        // Вход в режим HARD
        if(menuCursor == 2) { menuCursor = 21; TRY = 0; shootScore = 0; shootIntervalCursor = 0; flagDraw  = true; }
      }
      irrecv.resume();
    }
    if(globalState == 0) {
      ledBlinkTimer = millis();
      digitalWrite(pinGreenLed, HIGH);
      digitalWrite(pinRedLed, LOW);
      globalState = 1;
    }
    else if(globalState == 1) {
      if((millis() - ledBlinkTimer) > 500) {
        ledBlinkTimer = millis();
        digitalWrite(pinGreenLed, LOW);
        digitalWrite(pinRedLed, HIGH);
        globalState = 2;     
      }
    }
    else if(globalState == 2) {
      if((millis() - ledBlinkTimer) > 500) globalState = 0;
    }
    else if(globalState == 4) {
      flagDraw  = true;
      showMenu(menuCursor);
      digitalWrite(pinGreenLed, HIGH); digitalWrite(pinRedLed, LOW);
      beep(); beep();
      digitalWrite(pinGreenLed, LOW); digitalWrite(pinRedLed, HIGH);
      
      if(menuCursor == 100) globalState = 3;
      else if(menuCursor == 110) globalState = 5;
      else if(menuCursor == 210) {
        if(shootIntervalCursor < 4) shootIntervalCursor++;
        globalState = 7;
      }
      //flagDraw  = true;
      shootFlag = true;
    }
    else if(globalState == 5) {
      randomSeed(analogRead(0));
      mm_randomTime = random(3000, (randomTime * 1000)); ///////////////////////////////////////////////////////////
      Serial.print("randomTime="); Serial.println(mm_randomTime);
      mm_randomTimer = millis();
      digitalWrite(pinRedLed, HIGH);
      globalState = 6;
      shootFlag = true;
    }
    else if(globalState == 6) {
      // Если в течение выделенного таймслота нет попадания
      if((millis() - mm_randomTimer) > mm_randomTime) {
        if(shootScore > 0) shootScore--;
        shootFlag = false;
        beep_1();
        digitalWrite(pinRedLed, LOW); delay(200); 
        digitalWrite(pinRedLed, HIGH); delay(200); 
        digitalWrite(pinRedLed, LOW); delay(200);
        digitalWrite(pinRedLed, HIGH); delay(200);
        digitalWrite(pinRedLed, LOW); delay(200);
        flagDraw  = true; 
        globalState = 5;
      }
    }
    else if(globalState == 7) {
      Serial.print("Shoot time:"); Serial.println(shootTime * 1000);
      hm_Timer = millis();
      hm_Time = shootInterval[shootIntervalCursor] * 1000;
      Serial.print("shootInterval="); Serial.println(hm_Time);
      digitalWrite(pinRedLed, HIGH);
      globalState = 8;
      shootFlag = true;
      TRY++;
      flagDraw  = true;
    }
    else if (globalState == 8) {
      // Если общее время стрельбы ещё не вышло
      if((millis() - shootTimer) < (shootTime * 1000)) {
        // Если в течение выделенного таймслота нет попадания
        if((millis() - hm_Timer) > hm_Time) {
          if(shootScore > 0) shootScore--;
          beep_1();
          digitalWrite(pinRedLed, LOW); delay(200); 
          digitalWrite(pinRedLed, HIGH); delay(200); 
          digitalWrite(pinRedLed, LOW); delay(200);
          digitalWrite(pinRedLed, HIGH); delay(200);
          digitalWrite(pinRedLed, LOW); delay(200);
          flagDraw  = true;
          globalState = 7;
          if(shootIntervalCursor > 0) shootIntervalCursor--;
        }
      }
      else { // Если прошло время, отведённое на стрельбу
        digitalWrite(pinRedLed, LOW); 
        digitalWrite(pinRedLed, LOW);
        shootFlag = false;
        flagDraw  = true;
        globalState = 100;
        Serial.println("HARD MODE-TIME OUT");
      }
    }
    showMenu(menuCursor);
    if(menuCursor == -1) { menuCursor = 0; flagDraw  = true; }
  }
}
/* Функция-обработчик прерывания */
void shoot() {
  if(shootMode) {
    if(shootFlag) {
      shootFlag = false;
      globalState = 4;
      shootScore++;
    }
  }
}
